# -*- coding: utf-8 -*-
"""Imports for path specification factory."""

from dfvfs.path import apfs_container_path_spec
from dfvfs.path import apfs_path_spec
from dfvfs.path import bde_path_spec
from dfvfs.path import compressed_stream_path_spec
from dfvfs.path import cpio_path_spec
from dfvfs.path import cs_path_spec
from dfvfs.path import data_range_path_spec
from dfvfs.path import encoded_stream_path_spec
from dfvfs.path import encrypted_stream_path_spec
from dfvfs.path import ewf_path_spec
from dfvfs.path import ext_path_spec
from dfvfs.path import fake_path_spec
from dfvfs.path import fat_path_spec
from dfvfs.path import gpt_path_spec
from dfvfs.path import gzip_path_spec
from dfvfs.path import hfs_path_spec
from dfvfs.path import lvm_path_spec
from dfvfs.path import modi_path_spec
from dfvfs.path import mount_path_spec
from dfvfs.path import ntfs_path_spec
from dfvfs.path import os_path_spec
from dfvfs.path import path_spec
from dfvfs.path import phdi_path_spec
from dfvfs.path import qcow_path_spec
from dfvfs.path import raw_path_spec
from dfvfs.path import sqlite_blob_path_spec
from dfvfs.path import tar_path_spec
from dfvfs.path import tsk_path_spec
from dfvfs.path import tsk_partition_path_spec
from dfvfs.path import vhdi_path_spec
from dfvfs.path import vmdk_path_spec
from dfvfs.path import vshadow_path_spec
from dfvfs.path import xfs_path_spec
from dfvfs.path import zip_path_spec
